#!/bin/bash
# Run and Kill share's services defined in /usr/share/sambaedu/shares/%NAME%
# Works like Sys V init

## $Id$ ##


RC_SHARE=/usr/share/sambaedu/shares/shares.d
VERBOSE=0
LOGON=0
LOGOUT=0

function usage {
	echo "usage: $0 -v [-s|-k] share user computer address arch"
	echo "       -s : call logon scripts for share "
	echo "       -k : call logout scripts for share "
	echo "       -S : service name "
	echo "       -u : username "
	echo "       -m : machine name "
	echo "       -i : ip address "
	echo "       -a : client OS "
	echo "       -v : Verbose "
	exit $1;
}

function rc_logon {

	ERROR=0
	for script in "$RC_SHARE/$1/$5/"S*
	do
		[ ! -x $script ] && continue
		if [ "$VERBOSE" == "1" ]
		then
			echo "Running: $(basename $script) $2 $3 $4 $5" >> /var/log/samba/debug.log
		fi
		$script $2 $3 $4 $5
		if (( $? != 0 ))
		then
			ERROR=1
		fi
	done
	return $ERROR
}

function rc_logout {
	ERROR=0
	for script in "$RC_SHARE/$1/$5"/K*
	do
		[ ! -x $script ] && continue
		if [ "$VERBOSE" == "1" ]
		then
			echo "Running: $(basename $script) $2 $3 $4 $5" >> /var/log/samba/debug.log
		fi
		$script $2 $3 $4 $5
		if (( $? != 0 ))
		then
			ERROR=1
		fi
	done
	return $ERROR

}

while getopts ":skvS:u:m:i:a:" cmd
do
	case $cmd in	
		s) LOGON=1 ;;
		k) LOGOUT=1 ;;
		v) VERBOSE=1 ;;
		S) share=$OPTARG ;;
		u) user=$OPTARG ;;
		m) machine=$OPTARG ;;
		i) ip=$OPTARG ;;
		a) os=$OPTARG ;;
		?) echo "bad option!"
		   usage 1;;
	esac
done


# filtrage 
if [[ $user =~ ^.*_$ ]]
then
    exit 0
fi
u=$user
m=$machine
user=$(echo "$user" | tr 'A-Z' 'a-z' | sed "s/[\$_]$//")
machine=$(echo "$machine" | tr 'A-Z' 'a-z' |  sed "s/[\$_]$//")

echo "user: $u -> $user machine : $m -> $machine ip: $ip os: $os" >>/var/log/samba/debug.log

# on teste si windows fait des siennes
if [ "$user" == "$machine" ]
then
	exit 0
fi
if [ -z "$machine" ]
then
        $machine = $ip
fi
# ok, on transmet les infos sans transformation
if [ "$LOGON" == "1" ]
then
        rc_logon $share $user $machine $ip $os
elif [  "$LOGOUT" == "1" ]
then
        rc_logout $share $user $machine $ip $os
fi
exit $?
