#!/bin/bash

#variables
tempfile=`tempfile 2>/dev/null` || tempfile=/tmp/conf$$
dir_config="/etc/sambaedu"

config_ipse3_title="Adresse IP de Se3"

source /usr/share/sambaedu/includes/libs-common.sh
source /usr/share/sambaedu/includes/config.inc.sh


function show_title()
{
clear
BACKTITLE="Projet SambaEdu - https://www.sambaedu.org/"
WELCOME_TITLE="Synchronisation des données SE3 ---> SE4FS"
WELCOME_TEXT="Bienvenue sur l'outil de synchronisation des données du serveur de fichiers SambaEdu 4 File System avec le serveur SE3.

SambaEdu est un projet libre sous licence GPL vivant de la collaboration active des différents contributeurs

Soutenez l'association SambaÉdu !  https://www.sambaedu.org/Adherer-a-l-association"

dialog --ok-label Ok --backtitle "$BACKTITLE" --title "$WELCOME_TITLE" --msgbox "$WELCOME_TEXT" 15 70
}

# Fonction affichage du menu principal
function show_menu()
{
check_winbind
while :; do
dialog --backtitle "$BACKTITLE" --title "$WELCOME_TITLE" \
--menu "Choisissez l'action à effectuer" 15 70 7  \
"1" "Synchronisation de /home et /var/sambaedu" \
"2" "Synchronisation de /home uniquement" \
"3" "Synchronisation de /var/sambaedu" \
"4" "Corriger les droits sur les homes" \
2>$tempfile

    choice=`cat $tempfile`
    [ "$?" != "0" ] && exit 0
    case $choice in
            1)
            conf_se3ip
            copy_ssh_id
            sync_home
            sync_varse
            show_part "Terminé ! - Vous pouvez finaliser la migration en vous connectant à l'interface web du SE4FS et en important les GPO"
            echo ""
            exit 0
            ;;
            2)
            conf_se3ip
            copy_ssh_id
            sync_home
            show_part "Terminé ! - Appuyez sur entrée pour revenir au menu principal"
            read dummy
            ;;
            3)
            conf_se3ip
            copy_ssh_id
            sync_varse
            show_part "Terminé ! - Appuyez sur entrée pour revenir au menu principal"
            read dummy
            ;;
            4)
            correct_home
            show_part "Terminé ! - Appuyez sur entrée pour revenir au menu principal"
            read dummy
            ;;
            *) clear && exit 0
            ;;
            esac
done
}
# confirmation de la conf ip du se3
function conf_se3ip()
{

REPONSE=""
while [ "$REPONSE" != "yes" ]
do
    dialog --backtitle "$BACKTITLE" --title "IP du Se3" --inputbox "Veuillez saisir l'adresse IP du Se3" 15 70 $config_se3ip 2>$tempfile || erreur "Annulation"
    se3ip="$(cat $tempfile)"
                        
    confirm_title="Configuration de l'ip du SE3"
    confirm_txt="Adresse IP du serveur SE3 saisie :   $se3ip
    
Est-ce correct ?"	
    
    if (dialog --backtitle "$BACKTITLE" --title "$confirm_title" --yesno "$confirm_txt" 10 70) then
            REPONSE="yes"
            [ "$config_se3ip" != "$se3ip" ] && set_config sambaedu se3ip "$se3ip"
    else
            REPONSE="no"
    fi
done
}

function copy_ssh_id()
{
clear
show_part "Copie de la clé SSH sur le Se3"
show_info "Test de connexion sur $se3ip"
echo

ping -c 3 $se3ip
check_error
show_info "Tentative de connexion ssh root sur le Se3 et copie de la clé."
show_info "Veuillez saisir le mot de passe root s'il est demandé"

ssh-keyscan -H $se3ip >> ~/.ssh/known_hosts
eval $(ssh-agent -s)
ssh-add $dir_config/id_rsa
ssh-copy-id -i $dir_config/id_rsa.pub root@$se3ip
check_error
lcs_users_gid="$(ssh root@$se3ip getent group lcs-users | cut -d: -f3)"
admins_gid="$(ssh root@$se3ip getent group admins | cut -d: -f3)"
admin_gid="$(ssh root@$se3ip getent passwd admin | cut -d: -f3)"
}

function sync_home()
{
show_part "Traitement de la partition /home"
show_info "Synchronisation des données de /home...Patience"
rsync -aXxz --info=progress2 --exclude={profiles,netlogon,adminse3,_netlogon,_templates} root@$se3ip:/home/ /home/

if [ -e "/var/www/sambaedu" ]; then
    show_info "Traitement des profils sur /home : mise en conformité et correction des droits"
    cpt=1
    cpt_fin="$(ls /home | wc -l)"
     ls /home | while read useruid
    do
        show_progress
        if [ -d "/home/$useruid"  -a ! -L /home/$useruid ]; then
        
            # récup mozilla
            if [ -d /home/$useruid/profil/appdata/Mozilla ]
            then
                mkdir -p /home/$useruid/AppData/Roaming/Mozilla
                mv -f -t /home/$useruid/AppData/Roaming/Mozilla /home/$useruid/profil/appdata/Mozilla/* >/dev/null 2>&1
            fi
            
            if [ -d /home/$useruid/profil/appdata/Thunderbird ]
            then
                mkdir -p /home/$useruid/AppData/Roaming/Thunderbird
                mv -f -t /home/$useruid/AppData/Roaming/Thunderbird /home/$useruid/profil/appdata/Thunderbird/* >/dev/null 2>&1
            fi
            
            # récup bureau
            if [ -L "/home/$useruid/Bureau" ]
            then
                if [ "$user" = "admin" ]; then
                    rm -f /home/$useruid/profil/Bureau/Applications\ Wpkg.lnk
                fi
                rm -f /home/$useruid/Bureau
                mkdir -p /home/$useruid/Bureau
                rm -f /home/$useruid/profil/Bureau/Relance_connexion.bat
                mv -f -t /home/$useruid/Bureau /home/$useruid/profil/Bureau/* >/dev/null 2>&1
            fi
            
            # récup menu demarrer
            if [ -L "/home/$useruid/Demarrer" ]
            then
                rm -f /home/$useruid/Demarrer
                mkdir -p /home/$useruid/Demarrer
                mv -f -t /home/$useruid/Demarrer /home/$useruid/profil/Demarrer/* >/dev/null 2>&1
            fi
            
            # effacement dossier profil
            rm -fr /home/$useruid/profil
            
            
            
    #     echo "profil de $useruid traité<br>"
            
            
                if [ "$useruid" != "templates" -a "$useruid" != "profiles" ]; then
                    chmod -R 770 /home/$useruid
                    chown -R $useruid:Domain\ admins /home/$useruid
#                     prop=`stat -c%G /home/$useruid`
#                     if [ "$prop" != "domain admins" ]; then
#                         chgrp -R domain\ admins /home/$user 
#                     fi     
                fi
            
        fi
    done
    chmod 777 /home/profiles
    chgrp domain\ users /home/profiles
    
    
    
fi
}

function sync_varse()
{
show_part "Traitement de la partition /var/sambaedu"
show_info "Synchronisation des données de /var/sambaedu...Patience"
rsync -aXxAz --info=progress2 --exclude={save,wsusoffline,tmp_clonage*} root@$se3ip:/var/se3/ /var/sambaedu/

show_info "Récupération des ACLS du partage Progs sur Se3"
ssh root@$se3ip getfacl -R /var/se3/Progs | sed -e 's/admins$/domain\\040admins/ ; s/admins:/domain\\040admins:/ ; s/lcs-users/domain\\040users/ ; s/var\/se3/var\/sambaedu/ ; s/www-se3/www-admin/ ; /user:adminse3/d'> /var/sambaedu/acls_progs.txt
show_info "Adaptation des droits et restauration des ACLS sur SE4FS"
cd / 
setfacl --restore=/var/sambaedu/acls_progs.txt 

show_info "Récupération des ACLS du partage Docs sur Se3"
ssh root@$se3ip getfacl -R /var/se3/Docs | sed -e 's/admins$/domain\\040admins/ ; s/admins:/domain\\040admins:/ ; s/lcs-users/domain\\040users/ ; s/var\/se3/var\/sambaedu/ ; s/www-se3/www-admin/ ; /user:adminse3/d' > /var/sambaedu/list_acls_docs.txt
setfacl --restore=/var/sambaedu/list_acls_docs.txt 

show_info "Récupération des ACLS du dossier unattended sur Se3"
ssh root@$se3ip getfacl -R /var/se3/unattended | sed -e 's/admins$/domain\\040admins/ ; s/admins:/domain\\040admins:/ ; s/lcs-users/domain\\040users/ ; s/var\/se3/var\/sambaedu/ ; s/www-se3/www-admin/; /user:adminse3/d' > /var/sambaedu/list_acls_unattended.txt 
setfacl --restore=/var/sambaedu/list_acls_unattended.txt 2>/dev/null

cd -

if [ -e "/var/www/sambaedu" ]; then
    show_info "Correction des droits sur les classes"
    su www-admin -c "/usr/bin/php /usr/share/sambaedu/scripts/update_classe.php -c ALL" 2>&1| grep -v 'sudo' | sed 's/<[^>]*>//g ; s/&#224;/à/ ; s/-c ALL//' | grep partage

    show_info "Remise en place de Wpkg"
    rm -rf /var/sambaedu/unattended/install/os/*

    # Nettoyage :
    rm -f /var/sambaedu/unattended/install/wpkg/wpkg*
    rm -f /var/sambaedu/unattended/install/wpkg/initvars_se3.bat
    rm -f /var/sambaedu/unattended/install/wpkg/INSTALL
    
    rm -f /var/sambaedu/Progs/install/domscripts
    rm -f /var/sambaedu/Progs/install/repare*
    rm -rf /var/sambaedu/Progs/install/wpkg*
    rm -rf /var/sambaedu/Progs/install/paquet_post-clonage
    rm -rf /var/sambaedu/Progs/install/quitte_se3
    rm -rf /var/sambaedu/Progs/install/doc_post_clonage
    
    rm -f /var/sambaedu/Progs/ro/wpkg*
    rm -rf /var/sambaedu/Progs/ro/inventory
    
    rm -rf /var/sambaedu/unattended/install/bin
    rm -rf /var/sambaedu/unattended/install/computers
    rm -rf /var/sambaedu/unattended/install/dosbin
    rm -rf /var/sambaedu/unattended/install/drvbase
    rm -rf /var/sambaedu/unattended/install/lib
    rm -rf /var/sambaedu/unattended/install/linuxaux
    rm -rf /var/sambaedu/unattended/install/oldunattended
    rm -rf /var/sambaedu/unattended/install/scripts
    rm -rf /var/sambaedu/unattended/install/tools
        
    chmod 775 -R /var/sambaedu/unattended/install/wpkg/
    apt -q update
    if LC_ALL=C apt-get -s install sambaedu-wpkg | grep -q newest ; then
        apt install sambaedu-wpkg --reinstall
    else
        apt install sambaedu-wpkg 
    fi
    show_info "Completion de la base de Wpkg - Patience, c'est long !"
    
    su www-admin -c "/usr/bin/php /var/www/sambaedu/wpkg/wpkg_ldap_update.php >/dev/null 2>&1"
    cd /var/www/sambaedu/wpkg/transfert
    su www-admin -c "/usr/bin/php wpkg_app.php"
    su www-admin -c "/usr/bin/php wpkg_profile.php"
    su www-admin -c "/usr/bin/php wpkg_time.php"
    
    setfacl -R -m user::rwx,user:www-admin:rwx,user:admin:rwx,group::r-x,group:domain\\040admins:r-x,group:domain\\040computers:r-x,mask::rwx,other::---,default:user::rwx,default:user:www-admin:rwx,default:user:admin:rwx,default:group::r-x,default:group:domain\\040admins:r-x,default:group:domain\\040computers:r-x,default:mask::rwx,default:other::--- /var/sambaedu/unattended/install/wpkg

fi
# find /var/sambaedu/ -group $admins_gid -exec chgrp "Domain Admins" {} \;
# find /var/sambaedu/ -group $lcs_users_gid -exec chgrp "Domain Users" {} \;
# find /var/sambaedu/ -user $admin_gid -exec chown "admin" {} \;
}

# debut du script
show_title
show_menu
clear
exit 0
